package com.fjhx;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.entity.Config;
import com.fjhx.service.Service;
import com.fjhx.service.impl.ServiceImpl;
import com.fjhx.utils.BaseWebSocketClient;
import com.fjhx.utils.MyUtil;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;


public class MyMain {

    public static BaseWebSocketClient client = null;
    public static Config config;
    public static Service service = new ServiceImpl();

    public static void main(String[] args) throws IOException {

        System.out.println("Version： 2023-01-17 13:41");

        MyUtil.infoLog("程序启动中...");

        // 读取配置文件
        if (!readConfigFile()) {
            return;
        }

        // 开启webSocket连接
        connectWebSocket();

    }

    /**
     * 开启webSocket连接
     */
    public static void connectWebSocket() {

        try {
            client = BaseWebSocketClient.init(config.getWebSocketUrl());
            client.connect();
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

    }

    /**
     * 读取配置文件
     */
    public static boolean readConfigFile() throws IOException {
        // 获取配置文件路径
        File file = new File(MyUtil.dir + "\\config");
        if (!file.exists()) {
            MyUtil.errorLog("配置文件不存在");
            return false;
        }

        // 读取配置文件
        InputStreamReader read = new InputStreamReader(Files.newInputStream(file.toPath()), StandardCharsets.UTF_8);
        BufferedReader bufferedReader = new BufferedReader(read);
        StringBuilder builder = new StringBuilder();
        String lineTxt;
        while ((lineTxt = bufferedReader.readLine()) != null) {
            String trim = lineTxt.trim();
            if (!trim.startsWith("#")) {
                builder.append(trim);
            }
        }

        String json = builder.toString();

        try {
            config = JSONObject.parseObject(json, Config.class);
        } catch (Exception e) {
            MyUtil.errorLog("配置文件格式错误");
            return false;
        }

        return true;
    }

}
