package com.fjhx.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.MyMain;
import com.fjhx.constant.WebSocketConstant;
import com.fjhx.entity.ConnectConfig;
import com.fjhx.entity.MessageEntity;
import com.fjhx.enums.AgreementEnum;
import com.fjhx.enums.MessageTypeEnum;
import com.fjhx.service.Operation;
import com.fjhx.service.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

public class ServiceImpl implements Service {

    public static final Map<String, List<Operation>> operationMap = new ConcurrentHashMap<>();

    @Override
    public void handleMessage(MessageEntity messageEntity) {

        if (WebSocketConstant.OPERATING_UPPER_COMPUTER != messageEntity.getType()) {
            return;
        }

        // 数据
        JSONObject data = messageEntity.getData();
        String forwardUserId = ObjectUtil.defaultIfNull(data.getString(WebSocketConstant.FORWARD_USER_ID), "default");
        String forwardSessionId = ObjectUtil.defaultIfNull(data.getString(WebSocketConstant.FORWARD_SESSION_ID), "default");

        // 操作上位机类型
        Integer type = data.getInteger("type");

        switch (MessageTypeEnum.get(type)) {
            // case READ:
            //     read(sessionId, connectConfigList);
            //     break;
            case DISTINCT_READ:
                // 获取指令
                String instructions = data.getString("instructions");
                // ip配置集合
                List<ConnectConfig> connectConfigList = MyMain.config.getInstructionsMap().get(instructions);
                distinctRead(forwardUserId, forwardSessionId, ObjectUtil.clone(connectConfigList), data);
                break;
            case CLOSE:
                close(forwardSessionId);
                break;
        }

    }

    /**
     * 读取数据
     *
     * @param userId            用户id
     * @param sessionId         会话id
     * @param connectConfigList 连接配置
     */
    private void distinctRead(String userId, String sessionId, List<ConnectConfig> connectConfigList, JSONObject data) {
        List<Operation> operationList = newOperationList(sessionId);

        for (AgreementEnum agreementEnum : AgreementEnum.values()) {
            List<ConnectConfig> itemConnectConfigList = getConnectConfigList(agreementEnum, connectConfigList);
            if (itemConnectConfigList.size() > 0) {
                Operation operation = agreementEnum.getOperation();
                assert operation != null;
                operation.distinctRead(userId, sessionId, itemConnectConfigList, data);
                operationList.add(operation);
            }
        }

        if (operationList.size() == 0) {
            operationMap.remove(sessionId);
        }
    }

    /**
     * 关闭读取
     *
     * @param sessionId 会话id
     */
    private void close(String sessionId) {
        List<Operation> operationList = operationMap.get(sessionId);
        if (operationList != null) {
            operationList.forEach(Operation::close);
            operationMap.remove(sessionId);
        }
    }

    /**
     * 获取配置集合
     */
    private List<ConnectConfig> getConnectConfigList(AgreementEnum agreementEnum, List<ConnectConfig> connectConfigList) {
        return connectConfigList.stream()
                .filter(item -> agreementEnum.equals(AgreementEnum.get(item.getAgreement())))
                .collect(Collectors.toList());
    }

    /**
     * 获取本次会话id的操作集合
     */
    private List<Operation> newOperationList(String sessionId) {
        List<Operation> operationList = operationMap.computeIfAbsent(sessionId, k -> new ArrayList<>());

        if (operationList.size() > 0) {
            operationList.forEach(Operation::close);
            operationList.clear();
        }

        return operationList;
    }

}
