package com.fjhx.service.impl;

import cn.hutool.core.collection.ConcurrentHashSet;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.constant.UdpConstant;
import com.fjhx.entity.ConnectConfig;
import com.fjhx.service.Operation;
import com.fjhx.utils.MyUtil;
import com.fjhx.utils.ThreadPoolManager;
import com.fjhx.utils.Utility;
import lombok.SneakyThrows;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class UdpOperation implements Operation {

    /**
     * rfid集合
     */
    private final Set<String> rfidSet = new ConcurrentHashSet<>();

    /**
     * 设备连接配置
     */
    private List<ConnectConfig> connectConfigList = new ArrayList<>();

    /**
     * 监听开启状态标记
     */
    private boolean monitorFlag = true;


    @SneakyThrows
    @Override
    public void distinctRead(String userId, String sessionId, List<ConnectConfig> connectConfigList, JSONObject data) {
        this.connectConfigList = connectConfigList;

        for (ConnectConfig connectConfig : connectConfigList) {
            ThreadPoolManager.execute(() -> {

                // 开启设备读取rfid
                DatagramSocket datagramSocket = send(UdpConstant.START_READ_INSTRUCTION, connectConfig);
                startMonitor(datagramSocket, userId, sessionId, data);

            });
        }

    }

    private void startMonitor(DatagramSocket datagramSocket, String userId, String sessionId, JSONObject data) {

        new Thread(() -> {

            while (monitorFlag) {
                byte[] buf = new byte[128];
                DatagramPacket packet = new DatagramPacket(buf, buf.length);
                try {
                    datagramSocket.receive(packet);
                } catch (IOException e) {
                    e.printStackTrace();
                    break;
                }

                byte[] bytes = new byte[packet.getLength()];
                System.arraycopy(buf, 0, bytes, 0, packet.getLength());
                String str = Utility.bytes2HexString(bytes);

                // 截取rfid
                if (ObjectUtil.isEmpty(str) || str.length() < 20) {
                    continue;
                }

                String rfid = str.substring(
                        UdpConstant.UDP_RESULT_DATA_SUBSTRING_RFID_BEGIN_INDEX,
                        str.length() - UdpConstant.UDP_RESULT_DATA_SUBSTRING_RFID_END_INDEX);

                if (rfidSet.contains(rfid)) {
                    return;
                }
                rfidSet.add(rfid);

                MyUtil.errorLog("扫描到rfid标签:sessionId" + sessionId + ",rfid:" + rfid);
                sendRfidDate(userId, sessionId, rfid, data);
            }

        }).start();

    }

    @SneakyThrows
    @Override
    public void close() {

        for (ConnectConfig connectConfig : connectConfigList) {
            // 关闭设备读取rfid
            send(UdpConstant.STOP_READ_INSTRUCTION, connectConfig);
        }

        monitorFlag = false;
    }


    @SneakyThrows
    private DatagramSocket send(String instruction, ConnectConfig connectConfig) {
        byte[] convert2HexArray = Utility.convert2HexArray(instruction);
        DatagramPacket packets = new DatagramPacket(convert2HexArray, convert2HexArray.length,
                InetAddress.getByName(connectConfig.getIp()), connectConfig.getPort());

        DatagramSocket socket = new DatagramSocket(UdpConstant.DEFAULT_BINDING_PORT);
        socket.send(packets);

        return socket;
    }

}
